local mod = ...



-- Function: Sort a table by column "dist"
function SortByDist(row1, row2)
    return row1.dist < row2.dist
end



-- Training Workplace ---------------------------------------------------------------------------------------
-- Used to find all workplaces that already have a training object

local COMP_GUILD_WORKPLACE = {
	TypeName = "COMP_GUILD_WORKPLACE",
	ParentType = "COMPONENT",
	Properties = {
        { Name = "Type", Type = "string", Default = nil, Flags = { "SAVE_GAME" } },
        { Name = "TimerTraining", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
        { Name = "TimerLevy", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
        -- timer = 0 means it is ready to be used
        -- timer > 0 means it cannot be used until it has counted down to zero
    }
}

-- Register custom Class
mod:registerClass(COMP_GUILD_WORKPLACE)




-- *************************************************************************************************************
-- TRAINING Controller -------------------------------------------------------------------------------------
-- Assign TRAINING generators to new workplaces
-- *************************************************************************************************************

local COMP_GUILD_CONTROLLER = {
	TypeName = "COMP_GUILD_CONTROLLER",
	ParentType = "COMPONENT",
	Properties = {
    }
}



-- OnEnabled
function COMP_GUILD_CONTROLLER:onEnabled()

    mod:log("COMP_GUILD_CONTROLLER ______________________________________________________________________________________")

    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")


    -- Process workplaces on load
    self:processWorkplaces()


    -- Weekly event
    event.register(self, compMainGameLoop.ON_NEW_WEEK, function()
        self:processWorkplaces()
    end)

    -- Daily event
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()
        self:updateTimers()
    end)

end


-- Process Workplaces
function COMP_GUILD_CONTROLLER:processWorkplaces()

    -- Find all workplaces
    self:getLevel():getComponentManager("COMP_WORKPLACE"):getAllEnabledComponent():forEach(function(compWork)

        local objWork = compWork:getOwner()
        local compPart = objWork:getComponent("COMP_BUILDING_PART")
        local compBuilding = objWork:findFirstObjectWithComponentUp("COMP_BUILDING")
        local compGuildWorkplace = objWork:getComponent("COMP_GUILD_WORKPLACE")
        local countWorkers = #compWork.AssignedWorkerList
        if not countWorkers then countWorkers = 0 end

        -- Do not proceed if the workplace already has a TRAINING component, or if it is not a building part
        if compPart and compBuilding and (countWorkers > 0) and not compGuildWorkplace then

            -- Ensure the building part is already built
            if compPart:getIsBuilt() then

                -- Attach guild workplace component so that this code does not run again for this workplace
                compGuildWorkplace = objWork:getOrCreateComponent("COMP_GUILD_WORKPLACE")

                -- Discover what type of building this is
                local nameBuilding = compBuilding.AssetBuilding.Name

                local type = nil

                if nameBuilding == "BAKERY" then type = "BREADMAKERS"
                elseif nameBuilding == "BLACKSMITH" then type = "IRONWORKERS"
                elseif nameBuilding == "BREWERY" then type = "BREWERS"
                elseif nameBuilding == "COAL_HUT" then type = "WOODWORKERS"
                elseif nameBuilding == "LUMBER_CAMP" then type = "WOODWORKERS"
                elseif nameBuilding == "FORESTER_CAMP" then type = "WOODWORKERS"
                elseif nameBuilding == "HOP_FARM" then type = "BREWERS"
                elseif nameBuilding == "IRON_MINE" then type = "IRONWORKERS"
                elseif nameBuilding == "IRON_SMELTER" then type = "IRONWORKERS"
                elseif nameBuilding == "SAWMILL" then type = "WOODWORKERS"
                elseif nameBuilding == "SHEEP_FARM" then type = "CLOTHIERS"
                elseif nameBuilding == "DAIRY_FARM" then type = "MILKERS"
                elseif nameBuilding == "STONECUTTER_CAMP" then type = "STONEWORKERS"
                elseif nameBuilding == "STONEMASON_HUT" then type = "STONEWORKERS"
                elseif nameBuilding == "WEAVER_HUT" then type = "CLOTHIERS"
                elseif nameBuilding == "WHEAT_FARM" then type = "BREADMAKERS"
                elseif nameBuilding == "WINDMILL" then type = "BREADMAKERS"
                elseif nameBuilding == "FISHERS_HUT" then type = "FISHERS"
                elseif nameBuilding == "WEAPONSMITH" then type = "IRONWORKERS"
                elseif nameBuilding == "TAILORS_WORKSHOP" then type = "CLOTHIERS"
                elseif nameBuilding == "COOPERAGE" then type = "WOODWORKERS"
                elseif nameBuilding == "CHEESE_MAKER" then type = "MILKERS"
                elseif nameBuilding == "IRON_QUARRY" then type = "IRONWORKERS"
                -- elseif nameBuilding == "GOLD_QUARRY" then type = "JEWELLERS"
                elseif nameBuilding == "STONE_QUARRY" then type = "STONEWORKERS"
                -- elseif nameBuilding == "GOLD_SMELTER" then type = "JEWELLERS"
                elseif nameBuilding == "COMMON_WARES_WORKSHOP" then type = "WARES"
                elseif nameBuilding == "CANDLE_WORKSHOP" then type = "WARES"
                elseif nameBuilding == "SCULPTURE_WORKSHOP" then type = "STONEWORKERS"
                -- elseif nameBuilding == "STAINEDGLASS_WORKSHOP" then type = "CRAFTERS"
                -- elseif nameBuilding == "JEWELER_WORKSHOP" then type = "JEWELLERS"

                elseif nameBuilding == "FISHPOND_NAME" then type = "FISHERS"
                elseif nameBuilding == "BUILDING_POULTRYFARM_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_VEGETABLEFARM_NAME" then type = "VEGETABLES"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_BUTCHER_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_CHANDLER_NAME" then type = "WARES"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_COBBLER_NAME" then type = "LEATHERWORKERS"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_CREAMERY_NAME" then type = "MILKERS"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_CATTLE_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_SHEEP_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_GOAT_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_PIG_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "BUILDING_LIVESTOCKFARM_TANNER_NAME" then type = "LEATHERWORKERS"
                elseif nameBuilding == "JOINERWORKSHOP_NAME" then type = "WOODWORKERS"
                elseif nameBuilding == "BUILDING_FLAXFARM_NAME" then type = "CLOTHIERS"
                elseif nameBuilding == "BUILDING_CIDERPRESS_NAME" then type = "BREWERS"
                elseif nameBuilding == "BUILDING_ORCHARD_NAME" then type = "BREWERS"
                elseif nameBuilding == "BUILDING_UNDERGROUNDMINE_NAME" then type = "IRONWORKERS"
                elseif nameBuilding == "BUILDING_BASIC_POTTER_NAME" then type = "POTTERS"
                elseif nameBuilding == "BUILDING_CATTLEHERD_NAME" then type = "LIVESTOCK"
                elseif nameBuilding == "JK_FISHING_SHACKS_NAME" then type = "FISHERS"
                elseif nameBuilding == "BUILDING_BASIC_CLAYPIT_NAME" then type = "POTTERS"
                -- elseif nameBuilding == "BUILDING_BASIC_THATCHER_NAME" then type = "CRAFTERS"

                end

                -- The workplace matches a known TRAINING type
                if type then

                    -- Attach appropriate training type component
                    objWork:getOrCreateComponent("COMP_GUILD_TRAINING_" .. type)
                    mod:log("Guild training component created for building: " .. nameBuilding .. ", type " .. type)

                    -- Mark the generic component with the type, to be used in behaviour trees
                    compGuildWorkplace.Type = type

                    -- -- Create the training prefab and parent to the workplace
                    -- local objTraining = self:getLevel():createObject("PREFAB_GUILD_TRAINING_" .. type)
                    -- objTraining:setParent(objWork)

                    -- mod:log("Guild training object created for building: " .. nameBuilding .. ", type " .. type)

                else
                    mod:log("Unable to match building to guild training type: " .. nameBuilding)

                end

            end

        end

    end)

end



-- Update Timers
function COMP_GUILD_CONTROLLER:updateTimers()

    -- Find all guild training components
    self:getLevel():getComponentManager("COMP_GUILD_WORKPLACE"):getAllEnabledComponent():forEach(function(compGuildWorkplace)

        if compGuildWorkplace.TimerTraining > 0 then
            compGuildWorkplace.TimerTraining = compGuildWorkplace.TimerTraining - 1
        end

        if compGuildWorkplace.TimerLevy > 0 then
            compGuildWorkplace.TimerLevy = compGuildWorkplace.TimerLevy - 1
        end

    end)

end



-- Register Component
mod:registerClass(COMP_GUILD_CONTROLLER)



-- Attach component to the prefab manager
mod:registerPrefabComponent("PREFAB_MANAGER", { DataType = "COMP_GUILD_CONTROLLER" } )
