local mod = ...



-- Building Asset Processor -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/Guild1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Prefab -------------------------------------------------------------------------------------
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterMaleFront", "PREFAB_GUILD_MASTER_MALE_FRONT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterMaleBelt", "PREFAB_GUILD_MASTER_MALE_BELT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterMaleHat", "PREFAB_GUILD_MASTER_MALE_HAT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterFemaleFront", "PREFAB_GUILD_MASTER_FEMALE_FRONT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterFemaleBelt", "PREFAB_GUILD_MASTER_FEMALE_BELT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildMasterFemaleHat", "PREFAB_GUILD_MASTER_FEMALE_HAT")

mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyMaleFront", "PREFAB_GUILD_LEVY_MALE_FRONT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyMaleBelt", "PREFAB_GUILD_LEVY_MALE_BELT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyMaleHat", "PREFAB_GUILD_LEVY_MALE_HAT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyFemaleFront", "PREFAB_GUILD_LEVY_FEMALE_FRONT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyFemaleBelt", "PREFAB_GUILD_LEVY_FEMALE_BELT")
mod:registerAssetId("models/Guild1.fbx/Prefab/GuildLevyFemaleHat", "PREFAB_GUILD_LEVY_FEMALE_HAT")



-- Materials -------------------------------------------------------------------------------------
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Stone2", "MATERIAL_GUILD_STONE1")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Stone3", "MATERIAL_GUILD_STONE2")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Plaster1", "MATERIAL_GUILD_PLASTER1")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Plaster2", "MATERIAL_GUILD_PLASTER2")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Tile1", "MATERIAL_GUILD_TILE1")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Tile2", "MATERIAL_GUILD_TILE2")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Plank1", "MATERIAL_GUILD_PLANK1")
mod:registerAssetId("models/Guild1.fbx/Materials/Material.Plank2", "MATERIAL_GUILD_PLANK2")



-- Building Function Assignable -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "BUILDING_FUNCTION_ASSIGNABLE_GUILD_HALL",
	Name = "BUILDING_GUILD_NAME",
	AuthorizedFunctionList = "BUILDING_FUNCTION_LIST_GUILD_HALL",
    IsDescriptionOverride = false,
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "BUILDING_FUNCTION_LIST_GUILD_HALL",
	AssetFunctionList = {
	},
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "BUILDING_FUNCTION_ASSIGNABLE_GUILD_LEVY",
	Name = "BUILDING_GUILD_LEVY_NAME",
	AuthorizedFunctionList = "BUILDING_FUNCTION_LIST_GUILD_LEVY",
    IsDescriptionOverride = false,
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "BUILDING_FUNCTION_LIST_GUILD_LEVY",
	AssetFunctionList = {
	},
})



-- Register each type -------------------------------------------------------------------------------------
local function registerGuild(mesh, learning1, learning2, masteredJob, gameRules, taxBuildings)

    -- component for training destination
    local comp1 = {
        TypeName = "COMP_GUILD_TRAINING_" .. mesh,
        ParentType = "COMPONENT",
        Properties = {
        }
    }

    mod:registerClass(comp1)


    -- building function: masters
    local qty = 2
    if learning2 then qty = 1 end

    local learning = {
        { Resource = learning1, Quantity = qty },
    }

    if learning2 then
        table.insert(learning, { Resource = learning2, Quantity = qty })
    end

    mod:registerAsset({
        DataType = "BUILDING_FUNCTION_WORKPLACE",
        Id = "BUILDING_FUNCTION_GUILD_HALL_" .. mesh,
        Name = "BUILDING_FUNCTION_GUILD_HALL_" .. mesh .. "_NAME",
        WorkerCapacity = 4,
        HasMaximumInstance = true,
        MaximumInstanceAllowed = 1,
        RelatedJob = {
            Job = "JOB_GUILD_MASTER_" .. mesh,
            Behavior = "BEHAVIOR_GUILD_MASTER",
        },
        ResourceProduced = learning,
        GameRuleModifierList = gameRules,
    })

    mod:overrideAsset({
        Id = "BUILDING_FUNCTION_LIST_GUILD_HALL",
        AssetFunctionList = {
            Action = "APPEND",
            "BUILDING_FUNCTION_GUILD_HALL_" .. mesh,
        },
    })

    -- function: levy collectors
    mod:registerAsset({
        DataType = "BUILDING_FUNCTION_TAX_OFFICE",
        Id = "BUILDING_FUNCTION_GUILD_LEVY_" .. mesh,
        Name = "BUILDING_FUNCTION_GUILD_LEVY_" .. mesh .. "_NAME",
        WorkerCapacity = 2,
        HasMaximumInstance = true,
        MaximumInstanceAllowed = 2,
        IsPrivate = false,
        RelatedJob = {
            -- Job = "TAX_COLLECTOR",
            Job = "JOB_GUILD_LEVY_" .. mesh,
            Behavior = "BEHAVIOR_GUILD_LEVY",
        },
    })

    mod:overrideAsset({
        Id = "BUILDING_FUNCTION_LIST_GUILD_LEVY",
        AssetFunctionList = {
            Action = "APPEND",
            "BUILDING_FUNCTION_GUILD_LEVY_" .. mesh,
        },
    })

    -- job
    mod:registerAsset({
        DataType = "JOB",
        Id = "JOB_GUILD_MASTER_" .. mesh,
        JobName = "JOB_GUILD_MASTER_" .. mesh .. "_NAME",
        JobDescription = "JOB_GUILD_MASTER_DESC",
        IsLockedByDefault = true,
        DefaultBehavior = "BEHAVIOR_GUILD_MASTER",
        ProductionDelay = 60, -- default 30
        AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
        NeededMasteredJobList = { masteredJob },
        -- Hidden = true,
        CharacterSetup = {
            MaleFrontList = {
                "PREFAB_GUILD_MASTER_MALE_FRONT",
            },
            MaleHatList = {
                "PREFAB_GUILD_MASTER_MALE_HAT",
            },
            MaleBeltList = {
                "PREFAB_GUILD_MASTER_MALE_BELT",
            },
            FemaleFrontList = {
                "PREFAB_GUILD_MASTER_FEMALE_FRONT",
            },
            FemaleHatList = {
                "PREFAB_GUILD_MASTER_FEMALE_HAT",
            },
            FemaleBeltList = {
                "PREFAB_GUILD_MASTER_FEMALE_BELT",
            },
            IdleAnimation = "IDLE",
        },
    })

    mod:overrideAsset({
        Id = "NEWCOMER",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_MASTER_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "SERF",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_MASTER_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "COMMONER",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_MASTER_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "CITIZEN",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_MASTER_" .. mesh,
        },
    })


    -- job
    mod:registerAsset({
        DataType = "JOB",
        Id = "JOB_GUILD_LEVY_" .. mesh,
        JobName = "JOB_GUILD_LEVY_" .. mesh .. "_NAME",
        JobDescription = "JOB_GUILD_LEVY_DESC",
        IsLockedByDefault = true,
        DefaultBehavior = "BEHAVIOR_GUILD_LEVY",
        ProductionDelay = 30, -- default 30
        AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
        CharacterSetup = {
            MaleFrontList = {
                "PREFAB_GUILD_LEVY_MALE_FRONT",
            },
            MaleHatList = {
                "PREFAB_GUILD_LEVY_MALE_HAT",
            },
            MaleBeltList = {
                "PREFAB_GUILD_LEVY_MALE_BELT",
            },
            FemaleFrontList = {
                "PREFAB_GUILD_LEVY_FEMALE_FRONT",
            },
            FemaleHatList = {
                "PREFAB_GUILD_LEVY_FEMALE_HAT",
            },
            FemaleBeltList = {
                "PREFAB_GUILD_LEVY_FEMALE_BELT",
            },
            IdleAnimation = "IDLE",
        },
    })

    mod:overrideAsset({
        Id = "NEWCOMER",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_LEVY_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "SERF",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_LEVY_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "COMMONER",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_LEVY_" .. mesh,
        },
    })

    mod:overrideAsset({
        Id = "CITIZEN",
        CompatibleJobList = {
            Action = "APPEND",
            "JOB_GUILD_LEVY_" .. mesh,
        },
    })

    
end



-- Game Rules ---------------------------------------------------------------------------------------------------
local gameRulesStoneworkers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "STONE",
            "POLISHED_STONE",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesWoodworkers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "WOOD",
            "PLANK",
            "COAL",
            "RESOURCE_SHINGLE",
            "RESOURCE_FURNITURE",
            "BARREL",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesIronworkers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "IRON_ORE",
            "IRON",
            "TOOL",
            "SWORD",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesClothiers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "RESOURCE_FLAX",
            "RESOURCE_SHEEP",
            "WOOL",
            "WOOLEN_CLOTH",
            "CLOTHES",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesBreadmakers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "WHEAT",
            "FLOUR",
            "BREAD",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesMilkers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "MILK",
            "CHEESE",
            "BV1_BUTTER",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesVegetables = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "RESOURCE_VEGETABLE",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesLivestock = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "RESOURCE_CATTLE",
            "RESOURCE_PIG",
            "RESOURCE_GOAT",
            "BV1_EGG",
            "RESOURCE_POULTRY",
            "BV1_MEAT",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesFishers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "FISH",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesBrewers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "HOP",
            "BEER",
            "RESOURCE_CIDER",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesLeatherworkers = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "RESOURCE_HIDE",
            "LEATHER",
            "RESOURCE_SHOES",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesPotters = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "RESOURCE_CLAY",
            "RESOURCE_POTTERY",
            "RESOURCE_TILE",
        },
        YieldPercentage = 1.1,
    },
}

local gameRulesWares = {
    {
        DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
        ResourceList = {
            "CANDLE",
            "COMMON_WARES",
        },
        YieldPercentage = 1.1,
    },
}

-- local gameRulesGoldsmiths = {
--     {
--         DataType = "GAME_RULE_MODIFIER_CROP_YIELDS",
--         ResourceList = {
--             "GOLD_ORE",
--             "GOLD_BARS",
--             "JEWELRY",
--         },
--         YieldPercentage = 1.1,
--     },
-- }


-- Taxable building list
local taxBuildingsStoneworkers = {
    "BUILDING_STONECUTTER_CAMP",
    "BUILDING_STONEMASON_HUT",
    "BUILDING_QUARRY_STONE_DEPOSIT",
}

local taxBuildingsWoodworkers = {
    "BUILDING_WOODCUTTER_CAMP",
    "BUILDING_FORESTER",
    "BUILDING_COAL_HUT",
    "BUILDING_SAWMILL",
    "BUILDING_COOPERAGE",
}
if foundation.isModLoaded("4a8d3ff2-60af-4c75-8950-33e990975862") then table.insert(taxBuildingsWoodworkers, "JOINERWORKSHOP_NAME") end -- Furniture mod

local taxBuildingsIronworkers = {
    "BUILDING_BLACKSMITH",
    "BUILDING_QUARRY_IRON_DEPOSIT",
    "BUILDING_WEAPONSMITH",
    "BUILDING_IRON_SMELTER",
}
if foundation.isModLoaded("15ab204f-39cd-474d-9464-3463de3a2be2") then table.insert(taxBuildingsIronworkers, "BUILDING_UNDERGROUNDMINE_NAME") end -- Underground Mine mod

local taxBuildingsClothiers = {
    "BUILDING_SHEEP_FARM",
    "BUILDING_WEAVER_HUT",
    "BUILDING_TAILOR_WORKSHOP",
}
if foundation.isModLoaded("25402f80-c479-4c9b-a1c0-2bc7eedf17a1") then table.insert(taxBuildingsClothiers, "BUILDING_FLAXFARM_NAME") end -- Flax mod
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then table.insert(taxBuildingsClothiers, "BUILDING_LIVESTOCKFARM_SHEEP_NAME") end -- Livestock mod

local taxBuildingsBreadmakers = {
    "BUILDING_BAKERY",
    "BUILDING_WHEAT_FARM",
    "BUILDING_WINDMILL",
}

local taxBuildingsMilkers = {
    "BUILDING_DAIRY_FARM",
    "BUILDING_CHEESE_MAKER",
}
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then table.insert(taxBuildingsMilkers, "BUILDING_LIVESTOCKFARM_CREAMERY_NAME") end

local taxBuildingsVegetables = {
}
if foundation.isModLoaded("c8b63172-5b6d-45eb-9fab-a40c8ecd9947") then table.insert(taxBuildingsVegetables, "BUILDING_VEGETABLEFARM_NAME") end -- Vegetable and Poultry mod

local taxBuildingsLivestock = {
}

if foundation.isModLoaded("c8b63172-5b6d-45eb-9fab-a40c8ecd9947") then table.insert(taxBuildingsLivestock, "BUILDING_POULTRYFARM_NAME") end -- Vegetable and Poultry mod
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then
    table.insert(taxBuildingsLivestock, "BUILDING_LIVESTOCKFARM_BUTCHER_NAME")
    table.insert(taxBuildingsLivestock, "BUILDING_LIVESTOCKFARM_CATTLE_NAME")
    table.insert(taxBuildingsLivestock, "BUILDING_LIVESTOCKFARM_GOAT_NAME")
    table.insert(taxBuildingsLivestock, "BUILDING_LIVESTOCKFARM_PIG_NAME")
end -- Livestock mod

local taxBuildingsFishers = {
    "BUILDING_FISHING_HUT",
}
if foundation.isModLoaded("a673e06c-5bfd-4325-bd33-398fbd9e5531") then table.insert(taxBuildingsFishers, "JK_FISHING_SHACKS_NAME") end -- Fishing shacks mod
if foundation.isModLoaded("0e119c21-edfd-4e2e-a079-0678b38c85c0") then table.insert(taxBuildingsFishers, "FISHPOND_NAME") end -- Fish pond mod

local taxBuildingsBrewers = {
    "BUILDING_HOP_FARM",
    "BUILDING_BREWERY",
}
if foundation.isModLoaded("f252f14d-ca1d-4d02-97ca-46581d4b567d") then table.insert(taxBuildingsBrewers, "BUILDING_ORCHARD_NAME") end -- Orchard mod
if foundation.isModLoaded("f252f14d-ca1d-4d02-97ca-46581d4b567d") then table.insert(taxBuildingsBrewers, "BUILDING_CIDERPRESS_NAME") end -- Orchard mod

local taxBuildingsLeatherworkers = {
}
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then
    table.insert(taxBuildingsLeatherworkers, "BUILDING_LIVESTOCKFARM_COBBLER_NAME")
    table.insert(taxBuildingsLeatherworkers, "BUILDING_LIVESTOCKFARM_TANNER_NAME")
end -- Livestock mod

local taxBuildingsPotters = {
}
if foundation.isModLoaded("d757c3ab-0148-4290-aa51-5559329f5640") then
    table.insert(taxBuildingsPotters, "BUILDING_BASIC_POTTER_NAME")
    table.insert(taxBuildingsPotters, "BUILDING_BASIC_CLAYPIT_NAME")
end -- Basic Materials

local taxBuildingsWares = {
    "BUILDING_COMMON_WARES_WORKSHOP",
    "CANDLE_WORKSHOP",
}
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then
    table.insert(taxBuildingsWares, "BUILDING_LIVESTOCKFARM_CHANDLER_NAME")
end -- Livestock mod



-- Register guilds ---------------------------------------------------------------------------------------------
registerGuild("STONEWORKERS", "RESOURCE_LEARNING_MATHEMATICS", nil, "STONEMASON", gameRulesStoneworkers, taxBuildingsStoneworkers)
registerGuild("WOODWORKERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_MATHEMATICS", "CARPENTER", gameRulesWoodworkers, taxBuildingsWoodworkers)
registerGuild("IRONWORKERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_MATHEMATICS", "BLACKSMITH", gameRulesIronworkers, taxBuildingsIronworkers)
registerGuild("CLOTHIERS", "RESOURCE_LEARNING_MATHEMATICS", nil, "TAILOR", gameRulesClothiers, taxBuildingsClothiers)
registerGuild("BREADMAKERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_PLANTS", "BAKER", gameRulesBreadmakers, taxBuildingsBreadmakers)
registerGuild("MILKERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_ANIMALS", "CHEESE_MAKER", gameRulesMilkers, taxBuildingsMilkers)
if foundation.isModLoaded("c8b63172-5b6d-45eb-9fab-a40c8ecd9947") then registerGuild("VEGETABLES", "RESOURCE_LEARNING_PLANTS", nil, "JOB_VEGETABLEFARM_FARM", gameRulesVegetables, taxBuildingsVegetables) end
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then registerGuild("LIVESTOCK", "RESOURCE_LEARNING_ANIMALS", nil, "JOB_LIVESTOCKFARM_BUTCHER", gameRulesLivestock, taxBuildingsLivestock) end
registerGuild("FISHERS", "RESOURCE_LEARNING_ANIMALS", nil, "FISHERMAN", gameRulesFishers, taxBuildingsFishers)
registerGuild("BREWERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_PLANTS", "BREWER", gameRulesBrewers, taxBuildingsBrewers)
if foundation.isModLoaded("f6361b64-05d3-4ab5-a2db-b3b60dc4d307") then registerGuild("LEATHERWORKERS", "RESOURCE_LEARNING_ALCHEMY", "RESOURCE_LEARNING_ANIMALS", "JOB_LIVESTOCKFARM_COBBLER", gameRulesLeatherworkers, taxBuildingsLeatherworkers) end
if foundation.isModLoaded("d757c3ab-0148-4290-aa51-5559329f5640") then registerGuild("POTTERS", "RESOURCE_LEARNING_ALCHEMY", nil, "JOB_BASIC_POTTER", gameRulesPotters, taxBuildingsPotters) end
registerGuild("WARES", "RESOURCE_LEARNING_ALCHEMY", nil, "CANDLE_MAKER", gameRulesWares, taxBuildingsWares)



-- Building -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_GUILD",
	Name = "BUILDING_GUILD_NAME",
	Description = "BUILDING_GUILD_DESC",
	BuildingType = "ADMINISTRATION",
	AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
	BuildingPartSetList = {
		{
			Name = "BUILDING_PART_SETLIST_GUILD1_NAME",
			BuildingPartList = {
				"BUILDING_PART_GUILD_HALL1",
				"BUILDING_PART_GUILD_HALL_BALCONY1",
				"BUILDING_PART_GUILD_HALL_EXTENSION1",
				"BUILDING_PART_GUILD_HALL_MARKET1",
				"BUILDING_PART_GUILD_HALL_DOOR1",
			},
        },
		{
			Name = "BUILDING_PART_SETLIST_GUILD2_NAME",
			BuildingPartList = {
				"BUILDING_PART_MARKET_FOOD_STALL",
				"BUILDING_PART_MARKET_GOODS_STALL",
				"BUILDING_PART_MARKET_LUXURY_STALL",
				"BUILDING_PART_MARKET_BARRELS",
			},
        },
    },
    RequiredPartList = {
        {
            Category = "CORE",
            Min = 1,
        },
        {
            Category = "DOOR",
            Min = 1,
        },
    },
    VillagerRequired = { Status = "CITIZEN", Quantity = 1 },
	MaterialSetList = {
		{
			SetName = "MATERIAL_SET_GUILD1_NAME",
			MaterialList = {
				"MATERIAL_GUILD_STONE1",
				"MATERIAL_GUILD_PLASTER1",
				"MATERIAL_GUILD_TILE1",
				"MATERIAL_GUILD_PLANK1",
			},
		},
		{
			SetName = "MATERIAL_SET_GUILD2_NAME",
			MaterialList = {
				"MATERIAL_GUILD_STONE2",
				"MATERIAL_GUILD_PLASTER2",
				"MATERIAL_GUILD_TILE2",
				"MATERIAL_GUILD_PLANK2",
			},
		},
	},
})



-- Grounding -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Guild1.fbx/Prefab/GuildHall.Extension.Part.001/Door.005", { DataType = "COMP_GROUNDED", AutoDisable = true, })



-- Collider Platform -------------------------------------------------------------------------------------
-- mod:configurePrefabFlagList("models/Guild1.fbx/Prefab/GuildHall.Balcony.Part.001", { "PLATFORM" })



-- Attach Nodes (force attached parts to build after the main building parts) --------------------------------
mod:registerPrefabComponent("models/Guild1.fbx/Prefab/GuildHall.Extension.Part.001/Attach_Side.003", { DataType = "COMP_BUILDING_ATTACH_NODE", AttachNodeTypeList = { "SIDE", "BUILD_ORDER_DEPENDENCY" }})
mod:registerPrefabComponent("models/Guild1.fbx/Prefab/GuildHall.Balcony.Part.001/Attach_External.001", { DataType = "COMP_BUILDING_ATTACH_NODE", AttachNodeTypeList = { "EXTERNAL", "BUILD_ORDER_DEPENDENCY" }})
mod:registerPrefabComponent("models/Guild1.fbx/Prefab/GuildHall.Door.Part.001/Attach_Door.005", { DataType = "COMP_BUILDING_ATTACH_NODE", AttachNodeTypeList = { "DOOR", "BUILD_ORDER_DEPENDENCY" }})



-- Building Parts: Halls -------------------------------------------------------------------------------------

-- Hall
local function registerHall(index)

    -- Prefab
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Part.00" .. index, "PREFAB_GUILD_HALL" .. index)
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Part.00" .. index .. ".Constr", "PREFAB_GUILD_HALL" .. index .. "_CONSTR")

    -- Building Part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL" .. index,
        Name = "BUILDING_PART_GUILD_HALL_NAME",
        Description = "BUILDING_PART_GUILD_HALL_DESC",
        Category = "CORE",
        AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_GUILD_HALL",
        Capacity = 4,
        ConstructionVisual = "PREFAB_GUILD_HALL" .. index .. "_CONSTR",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_GUILD_HALL" .. index,
            MiniatureConfig = {
                ClipUnderGround = true,
                GroupHeight = 0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0,
                CameraPitchOffset = 0.0
            },
        },
        BuildingZone = {
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( {7, 9}, {0.0, 0.0} ),
                    Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
                },
            },
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 100 },
            },
            UpkeepCost = {
                { Resource = "GOLD_COINS", Quantity = 80 }
            },
            ResourceNeededList = {
                {
                    { Resource = "STONE", Quantity = 10 },
                    { Resource = "WOOD", Quantity = 20 },
                    { Resource = "POLISHED_STONE", Quantity = 20 },
                    { Resource = "TOOL", Quantity = 10 },
                    { Resource = "PLANK", Quantity = 40 },
                },
            },
        },
    })

end

registerHall(1)



-- Building Part: Extension --------------------------------------------------------------------------------------
-- Serves as warehouse and granary

-- Warehouse
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_GUILD_HALL_STORAGE_WAREHOUSE",
    Name = "BUILDING_PART_GUILD_HALL_EXTENSION_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_WAREHOUSE",
        },
    },
    AssetBuildingFunction = "BUILDING_FUNCTION_WAREHOUSE",
})

-- Granary
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_GUILD_HALL_STORAGE_GRANARY",
    Name = "BUILDING_PART_GUILD_HALL_EXTENSION_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_GRANARY",
        },
    },
    AssetBuildingFunction = "BUILDING_FUNCTION_GRANARY",
})

local function registerExtension(index)

    -- Prefab
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Extension.Part.00" .. index, "PREFAB_GUILD_HALL_EXTENSION" .. index)
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Extension.Part.00" .. index .. ".Constr", "PREFAB_GUILD_HALL_EXTENSION" .. index .. "_CONSTR")

    -- Attach type
    mod:registerPrefabComponent("PREFAB_GUILD_HALL_EXTENSION" .. index, { DataType = "COMP_BUILDING_PART", BuildingPartType = "SIDE" })

    -- Attach warehouse and granary to the parent prefab, which will shrink the storage buildings
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL_STORAGE_ASSEMBLAGE" .. index,
        Name = "BUILDING_PART_GUILD_HALL_EXTENSION_NAME",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
            CoreRandomBuildingPartList = { "PREFAB_GUILD_HALL_EXTENSION" .. index },
            MandatoryBuildingPartList = {
                { BuildingPart = "BUILDING_PART_GUILD_HALL_STORAGE_WAREHOUSE", OptionalAttachNodeString = "Attach_Major.Warehouse.00" .. index },
                { BuildingPart = "BUILDING_PART_GUILD_HALL_STORAGE_GRANARY", OptionalAttachNodeString = "Attach_Major.Granary.00" .. index },
            },
        },
    })

    -- Final building part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL_EXTENSION" .. index,
        Name = "BUILDING_PART_GUILD_HALL_EXTENSION_NAME",
        Description = "BUILDING_PART_GUILD_HALL_EXTENSION_DESC",
        IsOnlyAttached = true,
        ConstructionVisual = "PREFAB_GUILD_HALL_EXTENSION" .. index .. "_CONSTR",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
            RandomPartList = {
                "BUILDING_PART_GUILD_HALL_STORAGE_ASSEMBLAGE" .. index,
            },
            MiniatureConfig = {
                ClipUnderGround = true,
                GroupHeight = 0.0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0.0,
                CameraPitchOffset = 0.0
            },
        },
        BuildingZone = {
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( {3.8, 9}, {0.0, 0.0} ),
                    Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
                },
            },
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 50 },
            },
            UpkeepCost = {
                { Resource = "GOLD_COINS", Quantity = 25 }
            },
            ResourceNeededList = {
                {
                    { Resource = "STONE", Quantity = 10 },
                    { Resource = "WOOD", Quantity = 10 },
                    { Resource = "POLISHED_STONE", Quantity = 10 },
                    { Resource = "TOOL", Quantity = 2 },
                    { Resource = "PLANK", Quantity = 15 },
                },
            },
        },
    })
end

registerExtension(1)



-- Building Part: Covered Market --------------------------------------------------------------------------------------
local function registerMarket(index)

    -- Prefab
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Market.Part.00" .. index, "PREFAB_GUILD_HALL_MARKET" .. index)
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Market.Part.00" .. index .. ".Constr", "PREFAB_GUILD_HALL_MARKET" .. index .. "_CONSTR")

    -- Attach type
    mod:registerPrefabComponent("PREFAB_GUILD_HALL_MARKET" .. index, { DataType = "COMP_BUILDING_PART", BuildingPartType = "SIDE" })

    -- Building part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL_MARKET" .. index,
        Name = "BUILDING_PART_GUILD_HALL_MARKET_NAME",
        Description = "BUILDING_PART_GUILD_HALL_MARKET_DESC",
        IsOnlyAttached = true,
        ConstructionVisual = "PREFAB_GUILD_HALL_MARKET" .. index .. "_CONSTR",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_GUILD_HALL_MARKET" .. index,
            MiniatureConfig = {
                ClipUnderGround = true,
                GroupHeight = 0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0,
                CameraPitchOffset = 0.0
            },
        },
        BuildingZone = {
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( {3.8, 9}, {0.0, 0.0} ),
                    Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true },
                },
            },
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 30 },
            },
            UpkeepCost = {
                { Resource = "GOLD_COINS", Quantity = 15 }
            },
            ResourceNeededList = {
                {
                    { Resource = "WOOD", Quantity = 10 },
                    { Resource = "TOOL", Quantity = 2 },
                    { Resource = "PLANK", Quantity = 15 },
                },
            },
        },
    })
end

registerMarket(1)



-- Building Part: Balcony --------------------------------------------------------------------------------------
local function registerBalcony(index)

    -- Prefab
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Balcony.Part.00" .. index, "PREFAB_GUILD_HALL_BALCONY" .. index)
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Balcony.Part.00" .. index .. ".Constr", "PREFAB_GUILD_HALL_BALCONY" .. index .. "_CONSTR")

    -- Attach type
    mod:registerPrefabComponent("PREFAB_GUILD_HALL_BALCONY" .. index, { DataType = "COMP_BUILDING_PART", BuildingPartType = "EXTERNAL" })

    -- Building part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL_BALCONY" .. index,
        Name = "BUILDING_PART_GUILD_HALL_BALCONY_NAME",
        Description = "BUILDING_PART_GUILD_HALL_BALCONY_DESC",
        IsOnlyAttached = true,
        Category = "ROOM",
        AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_GUILD_LEVY",
        Capacity = 2,
        ConstructionVisual = "PREFAB_GUILD_HALL_BALCONY" .. index .. "_CONSTR",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_GUILD_HALL_BALCONY" .. index,
            MiniatureConfig = {
                ClipUnderGround = true,
                GroupHeight = 0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0,
                CameraPitchOffset = 0.0
            },
        },
        BuildingZone = {
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( {4.5, 1.25}, {0.0, -0.6} ),
                    Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false },
                },
            },
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 40 },
            },
            UpkeepCost = {
                { Resource = "GOLD_COINS", Quantity = 20 }
            },
            ResourceNeededList = {
                {
                    { Resource = "WOOD", Quantity = 5 },
                    { Resource = "TOOL", Quantity = 5 },
                    { Resource = "PLANK", Quantity = 15 },
                },
            },
        },
    })
end

registerBalcony(1)



-- Building Part: Door --------------------------------------------------------------------------------------
local function registerDoor(index)

    -- Prefab
    mod:registerAssetId("models/Guild1.fbx/Prefab/GuildHall.Door.Part.00" .. index, "PREFAB_GUILD_HALL_DOOR" .. index)

    -- Attach type
    mod:registerPrefabComponent("PREFAB_GUILD_HALL_DOOR" .. index, { DataType = "COMP_BUILDING_PART", BuildingPartType = "DOOR" })

    -- Building part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_GUILD_HALL_DOOR" .. index,
        Name = "BUILDING_PART_GUILD_HALL_DOOR_NAME",
        IsOnlyAttached = true,
        Category = "DOOR",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "PREFAB_GUILD_HALL_DOOR" .. index,
            MiniatureConfig = {
                ClipUnderGround = true,
                GroupHeight = 0,
                CameraPosition = { 0, 0, 0 },
                OrientationOffset = 0,
                CameraPitchOffset = 0.0
            },
        },
        Cost = {
            ResourceNeededList = {
                {
                    { Resource = "PLANK", Quantity = 2 },
                },
            },
        },
    })
end

registerDoor(1)
